package com.sun.java.swing.plaf.windows;

import java.awt.Dimension;
import javax.swing.plaf.basic.BasicGraphicsUtils;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.AbstractButton;
import sun.awt.AppContext;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Color;
import javax.swing.plaf.basic.BasicToggleButtonUI;

public class WindowsToggleButtonUI extends BasicToggleButtonUI
{
    protected int dashedRectGapX;
    protected int dashedRectGapY;
    protected int dashedRectGapWidth;
    protected int dashedRectGapHeight;
    protected Color focusColor;
    private static final Object WINDOWS_TOGGLE_BUTTON_UI_KEY;
    private boolean defaults_initialized;
    private transient Color cachedSelectedColor;
    private transient Color cachedBackgroundColor;
    private transient Color cachedHighlightColor;
    
    public WindowsToggleButtonUI() {
        this.defaults_initialized = false;
        this.cachedSelectedColor = null;
        this.cachedBackgroundColor = null;
        this.cachedHighlightColor = null;
    }
    
    public static ComponentUI createUI(JComponent component) {
        AppContext appContext = AppContext.getAppContext();
        WindowsToggleButtonUI value = (WindowsToggleButtonUI)appContext.get(WindowsToggleButtonUI.WINDOWS_TOGGLE_BUTTON_UI_KEY);
        if (value == null) {
            value = new WindowsToggleButtonUI();
            appContext.put(WindowsToggleButtonUI.WINDOWS_TOGGLE_BUTTON_UI_KEY, value);
            return value;
        }
        return value;
    }
    
    @Override
    protected void installDefaults(AbstractButton c) {
        super.installDefaults(c);
        if (!this.defaults_initialized) {
            String propertyPrefix = this.getPropertyPrefix();
            this.dashedRectGapX = (int)UIManager.get("Button.dashedRectGapX");
            this.dashedRectGapY = (int)UIManager.get("Button.dashedRectGapY");
            this.dashedRectGapWidth = (int)UIManager.get("Button.dashedRectGapWidth");
            this.dashedRectGapHeight = (int)UIManager.get("Button.dashedRectGapHeight");
            this.focusColor = UIManager.getColor(propertyPrefix + "focus");
            this.defaults_initialized = true;
        }
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            c.setBorder(xp.getBorder(c, WindowsButtonUI.getXPButtonType(c)));
            LookAndFeel.installProperty(c, "opaque", Boolean.FALSE);
            LookAndFeel.installProperty(c, "rolloverEnabled", Boolean.TRUE);
        }
    }
    
    @Override
    protected void uninstallDefaults(AbstractButton b) {
        super.uninstallDefaults(b);
        this.defaults_initialized = false;
    }
    
    protected Color getFocusColor() {
        return this.focusColor;
    }
    
    @Override
    protected void paintButtonPressed(Graphics graphics, AbstractButton abstractButton) {
        if (XPStyle.getXP() == null && abstractButton.isContentAreaFilled()) {
            Color color = graphics.getColor();
            Color background = abstractButton.getBackground();
            Color color2 = UIManager.getColor("ToggleButton.highlight");
            if (background != this.cachedBackgroundColor || color2 != this.cachedHighlightColor) {
                int red = background.getRed();
                int red2 = color2.getRed();
                int green = background.getGreen();
                int green2 = color2.getGreen();
                int blue = background.getBlue();
                int blue2 = color2.getBlue();
                this.cachedSelectedColor = new Color(Math.min(red, red2) + Math.abs(red - red2) / 2, Math.min(green, green2) + Math.abs(green - green2) / 2, Math.min(blue, blue2) + Math.abs(blue - blue2) / 2);
                this.cachedBackgroundColor = background;
                this.cachedHighlightColor = color2;
            }
            graphics.setColor(this.cachedSelectedColor);
            graphics.fillRect(0, 0, abstractButton.getWidth(), abstractButton.getHeight());
            graphics.setColor(color);
        }
    }
    
    @Override
    public void paint(Graphics g, JComponent c) {
        if (XPStyle.getXP() != null) {
            WindowsButtonUI.paintXPButtonBackground(g, c);
        }
        super.paint(g, c);
    }
    
    @Override
    protected void paintText(Graphics graphics, AbstractButton abstractButton, Rectangle rectangle, String s) {
        WindowsGraphicsUtils.paintText(graphics, abstractButton, rectangle, s, this.getTextShiftOffset());
    }
    
    @Override
    protected void paintFocus(Graphics g, AbstractButton abstractButton, Rectangle rectangle, Rectangle rectangle2, Rectangle rectangle3) {
        g.setColor(this.getFocusColor());
        BasicGraphicsUtils.drawDashedRect(g, this.dashedRectGapX, this.dashedRectGapY, abstractButton.getWidth() - this.dashedRectGapWidth, abstractButton.getHeight() - this.dashedRectGapHeight);
    }
    
    @Override
    public Dimension getPreferredSize(JComponent c) {
        Dimension preferredSize = super.getPreferredSize(c);
        AbstractButton abstractButton = (AbstractButton)c;
        if (preferredSize != null && abstractButton.isFocusPainted()) {
            if (preferredSize.width % 2 == 0) {
                ++preferredSize.width;
            }
            if (preferredSize.height % 2 == 0) {
                ++preferredSize.height;
            }
        }
        return preferredSize;
    }
    
    static {
        WINDOWS_TOGGLE_BUTTON_UI_KEY = new Object();
    }
}
